import logging

import sentry_sdk
from celery import shared_task
from django.conf import settings
from django.db import transaction
from django.utils import timezone
from dynamic_preferences.registries import global_preferences_registry

from shiftregister.messaging.outbound import send

from .models import Message, ShiftRegistration

global_preferences = global_preferences_registry.manager()

logger = logging.getLogger(__name__)


# cron task to send normal messages(reminders, changes) in batches
@shared_task
def send_messages():
    if not global_preferences["helper__send_sms"]:
        logger.info("sms disabled, not sending")
        return

    msgs = Message.objects.select_for_update().filter(sent_at__isnull=True)[
        : global_preferences["helper__sms_rate"] * 2
    ]
    with transaction.atomic():
        sent_msg_ids = []
        try:
            for msg in send(msg.as_outbound() for msg in msgs if not msg.sent_at):
                sent_msg_ids.append(msg.key)
        except Exception as e:
            logger.error(e)
            sentry_sdk.capture_exception(e)

        Message.objects.select_for_update().filter(id__in=sent_msg_ids).update(
            sent_at=timezone.now()
        )


# singlemessage so registration links arrive faster.
# will silently fail when messages are disabled.
# those messages will be picked up by the cron send later
# once we have decided to continue sending messages
@shared_task
def send_message(msgid, is_retry=False):
    if not global_preferences["helper__send_sms"]:
        return

    try:
        with transaction.atomic():
            msg = Message.objects.select_for_update().get(pk=msgid)
            if msg.sent_at:
                return

            list(send(msg.as_outbound()))

            msg.sent_at = timezone.now()
            msg.save()
    except Message.DoesNotExist:
        if not is_retry:
            logger.warning("message not found, retrying")
            send_message.apply_async((msgid, True), countdown=0.2)
        else:
            logger.error(f"message {msgid} not found in retry, giving up")


@shared_task
def send_reminders():
    regs = ShiftRegistration.objects.select_for_update().filter(
        reminder_sent=False,
        shift__start_at__lte=timezone.now()
        + global_preferences["helper__reminder_time"],
    )
    with transaction.atomic():
        for reg in regs:
            if reg.reminder_sent:
                continue

            try:
                reg.send_reminder()
            except Exception as e:
                logger.error(e)
                sentry_sdk.capture_exception(e)
