import json
from ast import literal_eval
from datetime import datetime, timezone

from clicksend_client import (
    ApiClient,
    Configuration,
    SMSApi,
    SmsMessage,
    SmsMessageCollection,
)
from clicksend_client.rest import ApiException
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from ..exceptions import OutboundMessageError
from ..message import Message, MessageType
from .abc import Receiver as BaseReceiver
from .abc import Sender as BaseSender

__all__ = ("Receiver", "Sender")

MAX_BATCH_SIZE = 1000  # see https://developers.clicksend.com/docs/rest/v3/#how-many-messages-can-i-send


class Receiver(BaseReceiver):
    fetch = None

    def handle(self, timestamp="", body="", message_id="", **kwargs):
        try:
            timestamp = int(timestamp)
        except ValueError:
            raise ValueError("invalid timestamp")

        if not kwargs.get("from", ""):
            raise ValueError("message has no sender")
        if not message_id:
            raise ValueError("empty message id")

        yield Message(
            message_id,
            sender=kwargs["from"],
            text=body,
            type=MessageType.INBOUND,
            created_at=datetime.fromtimestamp(timestamp, timezone.utc),
        )


class Sender(BaseSender):
    def __init__(self):
        for setting in (
            "clicksend_password",
            "clicksend_sender_id",
            "clicksend_username",
        ):
            try:
                settings.SMS_SETTINGS[setting]
            except KeyError:
                raise ImproperlyConfigured(
                    f"'{setting}' must be set in SMS_SETTINGS for ClickSend backend"
                )

        config = Configuration()
        config.username = settings.SMS_SETTINGS["clicksend_username"]
        config.password = settings.SMS_SETTINGS["clicksend_password"]

        self.client = ApiClient(config)

    def send(self, messages):
        messages = messages[:MAX_BATCH_SIZE]

        try:
            response = SMSApi(self.client).sms_send_post(
                SmsMessageCollection(
                    messages=[
                        SmsMessage(
                            _from=settings.SMS_SETTINGS["clicksend_sender_id"],
                            body=message.text.replace("\n", " "),
                            to=message.recipient,
                            source="shiftregister",
                            custom_string=message.key,
                        )
                        for message in messages
                    ]
                )
            )
        except ApiException as e:
            raise OutboundMessageError(f"{e.status} {e.reason}")

        data = literal_eval(response)["data"]

        for message in data.get("messages", []):
            if message["status"] == "SUCCESS":
                yield Message(
                    message["custom_string"],
                    recipient=message["to"],
                    sender=message["from"],
                    text=message["body"],
                    type=MessageType.OUTBOUND,
                    created_at=datetime.fromtimestamp(message["date"], timezone.utc),
                )
