from django.db import models
from django.db.models import Case, Count, F, Sum, When
from django.http import HttpResponse

from shiftregister.app.models import Helper, Message, Room, Shift, ShiftRegistration
from shiftregister.fallback.models import FallbackAssignment
from shiftregister.importer.models import Event


def metrics(request):
    worked_seconds_total = ShiftRegistration.objects.filter(
        state=ShiftRegistration.RegState.CHECKED_IN
    ).aggregate(sum=Sum("shift__duration"))["sum"]

    response = HttpResponse(
        "\n".join(
            (
                f"shiftregister_{name} {value}"
                for name, value in (
                    (
                        "asta_confirmed_total",
                        Helper.objects.filter(asta_confirmed=True).count(),
                    ),
                    (
                        "fallback_shifts",
                        Event.objects.filter(
                            deleted=False, calendar__needs_fallback=True
                        ).count(),
                    ),
                    (
                        "fallback_shifts_assigned",
                        FallbackAssignment.objects.aggregate(
                            count=Count("shift", distinct=True)
                        )["count"],
                    ),
                    (
                        "fallback_shifts_full",
                        Shift.with_reg_count()
                        .annotate(
                            real_required_helpers=Case(
                                When(
                                    required_helpers=0, then=F("room__required_helpers")
                                ),
                                default=F("required_helpers"),
                            ),
                            fallbackassignment_count=Count("fallbackassignment"),
                        )
                        .filter(
                            deleted=False,
                            reg_count__gte=F("real_required_helpers"),
                            fallbackassignment_count__gt=0,
                        )
                        .count(),
                    ),
                    ("helpers_total", Helper.objects.count()),
                    (
                        "helpers_confirmed_total",
                        Helper.objects.filter(number_validated=True).count(),
                    ),
                    (
                        "helpers_registered",
                        Helper.objects.annotate(
                            shift_count=Count(
                                Case(
                                    When(
                                        shiftregistration__state__in=[
                                            ShiftRegistration.RegState.REGISTERED,
                                            ShiftRegistration.RegState.CHECKED_IN,
                                        ],
                                        then=1,
                                    ),
                                    output_field=models.IntegerField(),
                                )
                            )
                        )
                        .filter(number_validated=True, shift_count__gte=1)
                        .count(),
                    ),
                    (
                        "helpers_helped_total",
                        Helper.objects.annotate(
                            shift_count=Count(
                                Case(
                                    When(
                                        shiftregistration__state__in=[
                                            ShiftRegistration.RegState.CHECKED_IN
                                        ],
                                        then=1,
                                    ),
                                    output_field=models.IntegerField(),
                                )
                            )
                        )
                        .filter(number_validated=True, shift_count__gte=1)
                        .count(),
                    ),
                    (
                        "helpers_required",
                        Shift.objects.filter(deleted=False)
                        .annotate(
                            real_required_helpers=Case(
                                When(
                                    required_helpers=0, then=F("room__required_helpers")
                                ),
                                default=F("required_helpers"),
                            )
                        )
                        .aggregate(sum=Sum("real_required_helpers"))["sum"]
                        or 0,
                    ),
                    (
                        "messages_sent_total",
                        Message.objects.all().count(),
                    ),
                    (
                        "shift_registrations",
                        ShiftRegistration.objects.filter(
                            state__in=(
                                ShiftRegistration.RegState.REGISTERED,
                                ShiftRegistration.RegState.CHECKED_IN,
                            )
                        ).count(),
                    ),
                    *(
                        (
                            f'shifts{{room="{room.name}"}}',
                            Shift.objects.filter(deleted=False, room=room).count(),
                        )
                        for room in Room.objects.all()
                    ),
                    *(
                        (
                            f'shifts_occupied{{room="{room.name}"}}',
                            Shift.with_reg_count()
                            .filter(deleted=False, reg_count__gte=1, room=room)
                            .count(),
                        )
                        for room in Room.objects.all()
                    ),
                    *(
                        (
                            f'shifts_full{{room="{room.name}"}}',
                            Shift.with_reg_count()
                            .annotate(
                                real_required_helpers=Case(
                                    When(
                                        required_helpers=0,
                                        then=F("room__required_helpers"),
                                    ),
                                    default=F("required_helpers"),
                                )
                            )
                            .filter(
                                deleted=False,
                                reg_count__gte=F("real_required_helpers"),
                                room=room,
                            )
                            .count(),
                        )
                        for room in Room.objects.all()
                    ),
                    (
                        "worked_seconds_total",
                        (
                            worked_seconds_total.total_seconds()
                            if worked_seconds_total
                            else 0.0
                        ),
                    ),
                    (
                        "worked_shifts_total",
                        ShiftRegistration.objects.filter(
                            state=ShiftRegistration.RegState.CHECKED_IN
                        ).count(),
                    ),
                )
            )
        )
    )
    response.headers["Content-Type"] = "text/plain; charset=utf-8"

    return response
